#!/bin/env pwsh

[CmdletBinding()]
Param(
    [switch] $ExpirementalDashboardUpdate,
    [switch] $Force
)

if ($IsLinux -and (-not $env:SUDO_UID))
{
    throw "this script must be run as root, aborting..."
}

if (-not (Get-Command "docker" -errorAction SilentlyContinue)) {
    throw "docker not installed, aborting...";
}

if (-not (Get-Command "docker-compose" -errorAction SilentlyContinue)) {
    throw "docker-compose not installed, aborting...";
}

Import-Module ./scripts/NetcoUtils


#Check .env.secret if there is an existing default admin password there, otherwise generate a new one
# force is 'y' if initPwd is new
 function GetAdminPasswordParams() {
    $retval = @{
        initPwd = ""
        force  = "n"
    }
    try {
        $envVars = Get-Content ./.env.secret -ErrorAction stop
        ForEach($_ in $envVars) {
            $res = [regex]::Match($_.trim(), '^INITIAL_ADMIN_PASSWORD\s*=\s*(.+)').groups[1].Value
            if($res.Length -gt 0) {
                Write-Host -ForegroundColor Green "INITIAL_ADMIN_PASSWORD found.`n"
                $retval['initPwd'] = $res
                return $retval
            }
        }
    } catch {
        # .env.secret didn't exist
        # Write-Host -ForegroundColor Red "INITIAL_ADMIN_PASSWORD not found.`n"
    }
    Write-Host -ForegroundColor Green "Generating initial admin password.`n"
    $retval['initPwd'] = (GenerateNewSecret);
    $retval['force'] = 'y'
    return $retval
}

 function GetForceParam() {

    try {
        $envVars = Get-Content ./.env.secret -ErrorAction stop
        ForEach($_ in $envVars) {
            $res = [regex]::Match($_.trim(), '^INITIAL_ADMIN_PASSWORD\s*=\s*(.+)').groups[1].Value
            if($res.Length -gt 0) {
                Write-Host -ForegroundColor Green "INITIAL_ADMIN_PASSWORD found.`n"
                return $res
            }
        }
    } catch {
        # .env.secret didn't exist
        Write-Host -ForegroundColor Red "INITIAL_ADMIN_PASSWORD not found.`n"
    }
    Write-Host -ForegroundColor Green "Generating initial admin password.`n"
    return (GenerateNewSecret);
}

function RunSetup() {

    $config = GetConfig
    $isInitialConfig = -not $config

    # Set default values
    $retentionPolicy = 'INF'
    $subnetAddress = '10.99.99.0/24'
    $serialPort = 'none'
    $listenAddress = '0.0.0.0'
    $listenPort = 80
    $selectedUserProfile = $env:USERPROFILE
    
    try {
        $selectedUserProfile = Get-Content ./.env.userprofile -ErrorAction stop
    } catch {
        # .env.username didn't exist        
    }


    $menu = @(
        ('1', "Allow access to local serial device (Linux)", [ref]$serialPort,
              ( "Enter the serial device such as '/dev/ttyUSB0' or '/dev/ttyS1' to pass to NETCO NMS.`n" +
                "To disable serial port access, type 'none'." ),
              { param($inp)
                if ( $inp -match '^/dev/tty.*$' )
                {
                    if ( Test-Path $inp -PathType leaf )
                        { return $inp }
                    else
                        { Write-Host "Device $inp does not exist" }
                }
                if ( $inp -match '^none$' )
                    { return $inp }
              },
              "Please enter 'none' or a valid /dev/tty* device node",
              (-not $IsLinux)
        ),
        ('2', "Listen address for NETCO NMS user interface", [ref]$listenAddress,
              ( "To limit NETCO NMS user interface to a specific NIC, enter its IPv4 address here.`n" +
                "To listen to all interfaces, enter '0.0.0.0'." ),
              { param($inp)
                if ( $inp -match '^([1-9]|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])(\.(\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])){3}$' )
                    { return $inp }
              },
              "This must be a valid IPv4 address or 0.0.0.0 to accept connections from any interface",
              $false
        ),
        ('3', "Listen port for NETCO NMS user interface", [ref]$listenPort,
              ( "" ),
              { param($inp)
                if ( $inp -match '^\d+$' -and [int]$inp -ge 1 -and [int]$inp -le 65535 )
                    { return $inp }
              },
              "This must be a valid port number (below 65535)",
              $false
        ),
        ('4', "Retention policy for monitoring data", [ref]$retentionPolicy,
              ( "Define how long to preserve monitoring data, for example '30d' or '52w'.`n" +
                "Units must be one of:`n    w = week`n    d = day`n    h = hour`n    m = minute`n    s = second`n" +
                "To preserve indefinitely, enter 'INF'. Minimum of one hour." ),
              { param($inp)
                if ( $inp -match '(^(?:\d+?)(w|d|h|m|s)$)|(^INF$)' )
                    { return $inp }
              },
              "The value must be either a number and unit [e.g. '105d'] or 'INF'.",
              $true
        ),
        ('5', "Internal Docker address space", [ref]$subnetAddress,
              ( "NETCO NMS uses an internal network to communicate across its components.`n" +
                "This should use an address space that is not in use in the company network.`n" +
                "Define which address space to use for internal Docker network in CIDR format`n[e.g. '192.168.0.0/16']" ),
              { param($inp)
                if ( $inp -match '^([1-9]|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])(\.(\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])){3}\/(3[0-2]|[1-2]\d|\d)$' )
                    { return $inp }
              },
              "Not a valid subnet. Please use correct CIDR notation such as '10.99.99.0/16'",
              $false
        ),
        ('6', "User profile path (Windows)", [ref]$selectedUserProfile,
              ( "Enter your Windows user profile path." ),
              { param($inp)
                if ( Test-Path -Path "$inp" ) #-notmatch '[T|t]empAdmin')
                { return $inp }
              },
              "The path doesn't exists. Please enter valid path to your Windows user profile.",
              $IsLinux
        )
    )

    do
    {
        Write-Host -ForegroundColor Yellow "Current NETCO NMS configuration`n"
        ForEach ($entry in $menu)
        {
            $entryNumber = $entry[0]
            $entryName = $entry[1]
            $entryReference = [ref]$entry[2]
            $entryInitialOnly = $entry[6]
            if ( $entryInitialOnly -and -not $isInitialConfig ) {
                Write-Host -ForegroundColor Gray "${entryNumber}. ${entryName}:" ${entryReference}.Value
            } else {
                Write-Host "${entryNumber}. ${entryName}:" ${entryReference}.Value
            }
        }
        Write-Host
        Write-Host "Enter the number to modify or press 'enter' to accept this configuration."

        $selection = Read-Host "Configure option"
        Write-Host
        ForEach ( $entry in $menu )
        {
            $entryNumber = $entry[0]
            $entryName = $entry[1]
            $entryReference = [ref]$entry[2]
            $entryDescription = $entry[3]
            $entryValidation = $entry[4]
            $entryInvalidText = $entry[5]
            $entryInitialOnly = $entry[6]

            if ( $selection -eq $entryNumber )
            {
                if ( $entryInitialOnly -and -not $isInitialConfig ) {
                    Write-Host "This option has been disabled. Please consult the manual for more information.`n"
                    continue
                }
                Write-Host -ForegroundColor Yellow "$entryName"
                if ( $entryDescription ) { Write-Host "$entryDescription" }
                do
                {
                    Write-Host "Enter new value or press 'enter' to leave unchanged."
                    $inp = Read-Host -Prompt $entryName
                    Write-Host
                    if ( !$inp ) { break; } # empty string => don't validate
                    $inp = $entryValidation.Invoke( $inp )
                    if ( !$inp -and $entryInvalidText ) { Write-Host -ForegroundColor Red "$entryInvalidText`n" }
                }
                 until ( $inp )

                # empty string? => don't assign
                if ( $inp ) { $entryReference.Value = $inp }
            }
        }
    }
    until ($selection -eq '')


    if (-not $IsLinux)
    {
        if ( "$selectedUserProfile" -match 'TempAdmin' ) {
                Write-Host -ForegroundColor Red "User profile $selectedUserProfile is your temporary admin profile, please set your real user profile path"
                Exit
        }
        if ( Test-Path -Path "$selectedUserProfile\.wslconfig" -PathType Leaf ) {
            #don't do anything
            Write-Host "WSL already configured"
			if ( -not(Test-Path -Path ".\.env.userprofile" -PathType Leaf)) {
				 Set-Content -Path '.env.userprofile' -Value ("$selectedUserProfile")
			}
        }
        else {          
            #copy .wslconfig in user folder  
            Write-Host -ForegroundColor Yellow "Configure your WSL"
            Copy-Item "./environments/.wslconfig" -Destination "$selectedUserProfile"
            Set-Content -Path '.env.userprofile' -Value ("$selectedUserProfile")
            Write-Host -ForegroundColor Yellow "Please restart your PC to complete WSL configuration and then start LocalUp.ps1 again"
            Start-Sleep -Seconds 2
            Exit
        }
    }


    $pwdData = (GetAdminPasswordParams);

    Add-Member -Force -InputObject $config -NotePropertyName General -NotePropertyValue @{
        RawRetentionPolicy = $retentionPolicy;
        ProcessedRetentionPolicy = $retentionPolicy;
        SubnetAddress = $subnetAddress;
        InitialAdminPassword = $pwdData['initPwd'];
        SerialPort = $serialPort;
        ListenAddress = $listenAddress;
        ListenPort = $listenPort;
        RegeneratePwd = $pwdData['force'];

    }

    UpdateConfig( $config )
}

RunSetup

function TestDockerForWindows()
{
    if(-not (Test-NetConnection 8.8.8.8 ).PingSucceeded)
    {
        throw "Could not connect internet (8.8.8.8), please connect internet. Offline install is not supported on windows environment."
    }

    $testPath = Resolve-Path "$PsScriptRoot/deps/nginx/"

    $dockerRunTest = Invoke-Expression "docker run -it -v '$($testPath):/var/test/' --rm busybox /bin/ls -- /var/test/"

    if(-not ($dockerRunTest -match "default.conf")) {
        throw "Mounting volume test with docker-for-windows failed, have you enabled drive sharing in docker configuration? Please see https://docs.docker.com/docker-for-windows/#shared-drives"
    }
}

if ( -not (Test-Path "$PsScriptRoot/offline/") )
{
    Get-Content $PSScriptRoot/deps/pull-secret.json | Out-String | docker login -u _json_key --password-stdin https://eu.gcr.io
}

if (-not $IsLinux)
{
    TestDockerForWindows
}

if (-not (Test-Path (ResolveConfigFilePath) -PathType Leaf))
{
    Write-Host "Initial configuration not done, running Setup.ps1"
    Invoke-Expression "./Setup.ps1"
}

./scripts/FixWindowsVsLinuxLineEndings.ps1
./scripts/FixLinuxPermissions.ps1

$config = GetConfig

$addresses = GetSubCidrAddresses $config.General.SubnetAddress 6
if ( $config.General.SerialPort -eq 'none' ) {
    $serialDev = '/dev/null'
} else {
    $serialDev = $config.General.SerialPort
}

(Get-Content .env.images) | Where-Object { $_.trim() -ne "" -And -not $_.StartsWith("#") } | Set-Content .env
Add-Content .env (
    "NETCO_SUBNET_ADDRESS_MONITORING=$($addresses[0])" +
    "`nNETCO_SUBNET_ADDRESS_DESIGN=$($addresses[1])" +
    "`nNETCO_SUBNET_ADDRESS_MISC=$($addresses[2])" +
    "`nNETCO_SUBNET_ADDRESS_DATA=$($addresses[3])" +
    "`nNETCO_SUBNET_ADDRESS_SNMPSIM=$($addresses[4])" +
    "`nNETCO_SUBNET_ADDRESS_SHARED=$($addresses[5])" +
    "`nSNMP_GW_IP_ADDRESS=$(GetIpFromRange $addresses[4] 3)" +
    "`nNETCO_SERIAL_PORT=" + $serialDev +
    "`nNETCO_LISTEN_ADDR=" + $config.General.ListenAddress +
    "`nNETCO_LISTEN_PORT=" + $config.General.ListenPort +
    "`nDEVICES_STRING=" +
    "`n`n# PLEASE NOTE: THIS FILE IS AUTOMATICALLY GENERATED`n# AND SHOULD NOT BE DIRECTLY EDITED.")

Set-Content -Path '.env.secret' -Value (
    "SECRET=$(GenerateNewSecret)`n"+
    "FORCE_INIT_PWD=$($config.General.RegeneratePwd)`n"+
    "INITIAL_ADMIN_PASSWORD=$($config.General.InitialAdminPassword)")


if ( $config.General.ListenAddress -eq "0.0.0.0" )
    { $serviceAddress = "http://localhost" }
else
    { $serviceAddress = "http://$($config.General.ListenAddress)" }

if ( $config.General.ListenPort -eq 80 )
    { $serviceAddress += "/" }
else
    { $serviceAddress += ":$($config.General.ListenPort)/" }

# Pulling separately!!!! 
#if (-not $IsLinux)
#{   
#    Write-Host "Pull images first"
#    Get-Content .env.images | Where-Object {$_.length -gt 0} | Where-Object {!$_.StartsWith("#")} | ForEach-Object {
#
#        $var = $_.Split('=',2).Trim()
#        New-Variable -Scope Script -Name $var[0] -Value $var[1]
#        docker pull $var[1] 
#    }
#}

Write-Host "Create containers"
docker compose --project-name netco_nms up -d

$rp_raw = $config.General.RawRetentionPolicy
$rp_processed = $config.General.ProcessedRetentionPolicy

$influxContainerName = docker ps --filter name=netco_nms.influxdb --format "{{.Names}}"

$response = ""
while ($response -notlike "*204*")
{
    $response = docker exec $influxContainerName curl -i -sf -o /dev/null -w "%{http_code}" -XGET http://localhost:8086/ping
    Write-Host "Waiting for database connection..."
    Start-Sleep -Seconds 5
}

$dbSchema = docker exec $influxContainerName curl -sb -XPOST --data-urlencode "q=SHOW RETENTION POLICIES ON netco" http://localhost:8086/query | ConvertFrom-Json
$policiesExists = $dbSchema.results.series -and ($dbSchema.results.series[0].values.Length -gt 1)

if ($policiesExists)
{
    docker exec $influxContainerName curl -i -sf -o /dev/null -XPOST --data-urlencode "q=ALTER RETENTION POLICY `"rp_raw`" ON `"netco`" DURATION $rp_raw REPLICATION 1" http://localhost:8086/query
    docker exec $influxContainerName curl -i -sf -o /dev/null -XPOST --data-urlencode "q=ALTER RETENTION POLICY `"rp_processed`" ON `"netco`" DURATION $rp_processed REPLICATION 1" http://localhost:8086/query
    Write-Host "Updated retention policies: rp_raw ($rp_raw), rp_processed ($rp_processed)"
}
else
{
    # Ensure database exists
    docker exec $influxContainerName curl -i -sf -o /dev/null -XPOST --data-urlencode "q=CREATE DATABASE `"netco`"" http://localhost:8086/query
    docker exec $influxContainerName curl -i -sf -o /dev/null -XPOST --data-urlencode "q=CREATE RETENTION POLICY `"rp_raw`" ON `"netco`" DURATION $rp_raw REPLICATION 1" http://localhost:8086/query
    docker exec $influxContainerName curl -i -sf -o /dev/null -XPOST --data-urlencode "q=CREATE RETENTION POLICY `"rp_processed`" ON `"netco`" DURATION $rp_processed REPLICATION 1" http://localhost:8086/query
    Write-Host "Created retention policies: rp_raw ($rp_raw), rp_processed ($rp_processed)"
}

if ($ExpirementalDashboardUpdate) {
    $dashboard = Invoke-RestMethod $serviceAddress/api/dashboards/uid/i77hsMniz
        -Method GET
        -Headers @{"Authorization"="Bearer eyJrIjoiazF5MGdVejVwQXlxeEZDbUJlejVSZTVQdnMwUHNXNTEiLCJuIjoiYXNkIiwiaWQiOjF9"}
        -ContentType "application/json"

    Invoke-RestMethod $serviceAddress/grafana/api/org/preferences
        -Method PUT
        -Body (@{ homeDashboardId = $dashboard.dashboard.id } | ConvertTo-Json)
        -Headers @{"Authorization"="Bearer eyJrIjoiazF5MGdVejVwQXlxeEZDbUJlejVSZTVQdnMwUHNXNTEiLCJuIjoiYXNkIiwiaWQiOjF9"}
        -ContentType "application/json"
}

Write-Host
Write-Host -ForeGround Green "Navigate to $($serviceAddress)"
if ($config.General.RegeneratePwd -eq 'y'){
    Write-Host -ForeGround Green "Initial admin login with username: 'admin', password: '$($config.General.InitialAdminPassword)'."
    Write-Host -ForeGround Green "Save this password or change it at User Management page"
}
else {
    #Write-Host -ForeGround Green "Use your old password for admin user"
}
